#include "includes/common_functions.inc"

// * User input * //
//uniform mediump sampler2D u_texture;
//uniform mediump sampler2D u_texture1;
uniform mediump sampler2D u_texture; // environment

// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 
// Ambient Color in RGB, intensity in A (contrast)
uniform lowp vec4 u_ambientLightColor;
uniform lowp vec4 u_rimcontrols;
	
uniform lowp vec4 u_materialDiffuse;
uniform lowp vec4 u_materialSpecular;
uniform lowp vec4 u_materialEmissive;
uniform lowp vec4 u_materialShininess;

// Special handling for 2D UI world...
uniform lowp vec4 u_uicontrol;
	
// Data IN FROM VERTEX SHADER
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;
//varying mediump vec4 v_vert_color;
//varying lowp vec2 v_texCoord0;
varying lowp vec3 v_camDir;

void main() {
	// NORMALIZE INPUT VALUES TO ENSURE COUNTER INTERPOLATION ISSUES
	mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	mediump vec3 V = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);
	mediump vec3 B = normalize(v_binormal);
	mediump vec3 T = normalize(v_tangent);
	
	// ALBEDO MAP  -----------------------------------------------------------------------------
	//lowp vec4 albedo = texture2D(u_texture, v_texCoord0);
	//albedo.rgb = pow(albedo.rgb, vec3(2.2, 2.2, 2.2));
	lowp vec4 albedo = u_materialDiffuse;
	
	// DOT PRODUCTS + REFLECTION VECTORS
	lowp float NdotV = dot(N, V);
	lowp float NdotL = (dot(N, L)+ 1.0) * 0.5;   // 1/2 LAMBERTIAN LIGHTING MODEL
	mediump vec3 R = reflect(-V, N*vec3(u_uicontrol.x, u_uicontrol.y, 1.0));
	lowp float NormalUp = u_uicontrol.z + u_uicontrol.w * (N.y + 1.0)*0.5;
	
	// LIGHTING CALCLUATIONS
	lowp vec3 dir_light = u_directionalLightColor.rgb * NdotL * u_directionalLightColor.a * albedo.a;
	lowp vec3 ambient_light = mix(u_ambientLightColor.rgb * vec3(0.8, 0.25, 0.1), u_ambientLightColor.rgb, NormalUp) * u_ambientLightColor.a * albedo.a;
	
	// GET METALLIC ROUGHNESS WETNESS FROM VERTEX COLOURS
	mediump vec3 mrw = u_materialSpecular.rgb; // g = specularity, 

	// SPEC ------------------  -----------------------------------------------------------------------------
	mediump vec3 specColour = mix(vec3(0.03, 0.03, 0.03), albedo.rgb * 2.0, mrw.r);
	
	highp vec2 sphericalCoords = vert_CalculateSphericalCoords(mrw.g, R);
	//highp vec2 sphericalCoords2 = (vec4(N,1.0) * u_viewMatrixInverse).xy * -0.5 + vec2(0.5); // better reflections?
	//sphericalCoords2.r = 0.25 * ((sphericalCoords2.r) + floor((3.0 * mrw.g) + 0.5)); // handle roughness
	//sphericalCoords = sphericalCoords2;
	
	//ENV SAMPLING
	lowp vec4 envMapTex = texture2D(u_texture, sphericalCoords);
	envMapTex = pow(envMapTex, vec4(2.2, 2.2, 2.2, 2.2));
	
	//Function CalculateEnvSpec accepts parameters (vec4 envMap, vec3 mrw, float NdotV, vec3 amb_color, vec3 spec_color)
	//Returns vec4 envSpec, env map colors in rgb, spec coefficient in a.
	mediump vec4 envSpec = CalculateEnvSpec(envMapTex, mrw, NdotV, ambient_light, specColour);
	
	mediump vec3 spec_BRDF = CalculateBRDF(NdotV, mrw.g, specColour, dir_light, envSpec.a, albedo.a );	
	

	// RIM LIGHTING   ---------------------------------------------------------------------------------
	lowp vec3 rimLight = mix(dir_light.rgb ,albedo.rgb , NdotL);
	rimLight *= pow(max(0.0001, 1.0 - NdotV), u_rimcontrols.y) * (u_uicontrol.y * N.y + 1.0) * u_rimcontrols.x * albedo.a * u_materialSpecular.a;
	
	// BUILD FRAGMENT OUTPUT  -------------------------------------------------------------------------
	//mediump vec4 returnValue = vec4(albedo.rgb *(dir_light + ambient_light) , 1.0);
	mediump vec4 returnValue = vec4(0.0);
	//returnValue.rgb += spec_BRDF.rgb * NormalUp;
	returnValue.rgb += envSpec.rgb;
	returnValue.rgb += rimLight;
	returnValue.rgb += mix(u_materialEmissive.rgb, vec3(0.0), clamp(length(returnValue.rgb),0.0,1.0)); // we only want to add emissive at darker values
	
	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = pow(returnValue.rgb, vec3(1.0/2.2, 1.0/2.2, 1.0/2.2));
	gl_FragColor = returnValue;
	//gl_FragColor = vec4(envMapTex);
	//gl_FragColor = vec4(sphericalCoords * vec2(3.0,1.0), 0.0, 1.0);
	
	//gl_FragColor = vec4(R, 1.0);
	//gl_FragColor = vec4(1.0-rimLight, 1.0);
}