#include "includes/common_functions.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

// * User input * //
uniform lowp sampler2D u_texture;
uniform lowp sampler2D u_texture1;
uniform lowp sampler2D u_texture2;


// Light matrix in world space
uniform highp mat4 u_directionalLightMatrix; 
// Dir Color in RGB , intensity in A
uniform lowp vec4 u_directionalLightColor; 
// Ambient Color in RGB, intensity in A (contrast)
uniform mediump vec4 u_ambientLightColor;

uniform lowp vec4 u_translucency;	

// Data IN FROM VERTEX SHADER
varying lowp vec3 v_ambient_colour;
varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;
varying lowp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec2 v_texCoord2;
varying lowp vec3 v_camDir;
varying lowp vec2 v_sphericalCoords;
varying lowp float v_closeDarken;
varying lowp vec2 v_intRef;

//varying lowp float v_fogIntensity;

void main() {
	// NORMALIZE INPUT VALUES TO ENSURE COUNTER INTERPOLATION ISSUES
	mediump vec3 L = normalize(-u_directionalLightMatrix[2].xyz);
	mediump vec3 V = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);

	// DOT PRODUCTS + REFLECTION VECTORS
	float NdotV = dot(N, V);
	float NdotL = max(0.0001, dot(L, N));
	float LdotV = (dot(L, V)+1.0) * 1.0;
	
	// LIGHTING CALCLUATIONS
	lowp vec3 dir_light = u_directionalLightColor.rgb * NdotL * u_directionalLightColor.a;

	// ALBEDO - DIFFUSE TEXTURE READ
	lowp vec4 albedo = texture2D(u_texture, v_texCoord0);
	albedo.rgb = albedo.rgb * albedo.rgb;
	
	// TRANSLUCENCY INSIDE - TEXTURE READ AND CALCULATIONS
	lowp vec4 refracted = texture2D(u_texture1, v_intRef);
	refracted.rgb *= (N*N);
	refracted.rgb = vec3(clamp((refracted.r + refracted.g + refracted.b), 0.0, 1.0));	
	lowp vec3 brighter = clamp( albedo.rgb + (u_translucency.x * albedo.rgb), 0.1, 0.9);		
	refracted.rgb = vec3(mix(albedo.rgb, brighter.rgb, (refracted.g *clamp((NdotV*NdotV)*(1.0-NdotL), 0.0, 1.0))));
	albedo.rgb=mix(albedo.rgb, refracted.rgb, albedo.a);

	float translucencyContribution = (LdotV + u_translucency.y) *albedo.a;
	
	// PHYSICALLY CORRECT SHADING
	lowp vec3 mrw = v_vert_color.rgb;

	// SPEC APPROXIMATION  
	lowp vec3 specColour = mix(vec3(0.03), albedo.rgb * 2.0, mrw.r);

	//ENV SAMPLING
	lowp vec4 envMapTex = texture2D(u_texture2, v_sphericalCoords);
	envMapTex = envMapTex * envMapTex;

	//Function CalculateEnvSpec accepts parameters (vec4 envMap, vec3 mrw, float NdotV, vec3 amb_color, vec3 spec_color)
	//Returns vec4 envSpec, env map colors in rgb, spec coefficient in a.
	lowp vec4 envSpec = CalculateEnvSpec(envMapTex, mrw, NdotV, v_ambient_colour, specColour);
	lowp vec3 spec_BRDF = CalculateBRDF(NdotV, mrw.g, specColour, dir_light, envSpec.a, albedo.a );
	
	// RIM LIGHTING   ---------------------------------------------------------------------------------
	lowp vec3 rimLight = mix(brighter.rgb ,albedo.rgb , NdotL);
	rimLight *= pow(max(0.0001, 1.0 - NdotV), u_translucency.w) * ( N.y + 1.0) * u_translucency.z ;

	// BUILD FRAGMENT OUTPUT  
	lowp vec4 returnValue = vec4(albedo.rgb * (dir_light + v_ambient_colour + translucencyContribution), 1.0);
	returnValue.rgb += spec_BRDF.rgb;
	returnValue.rgb += envSpec.rgb;
	returnValue.rgb += rimLight.rgb;

	// CHEAP FOG
	 lowp vec3 fogColour = u_fogColor.rgb;
	 returnValue.rgb = mix(returnValue.rgb, fogColour, v_vert_color.a);

	// MOVE BACK INTO sRGB COLOUR SPACE
	returnValue.rgb = sqrt(returnValue.rgb);

	// ADD COLOR tO CLOSEDARKEN
	returnValue.rgb = mix((closeDarkenColor(u_closeDarken)* returnValue.rgb), returnValue.rgb, v_closeDarken);
	gl_FragColor = returnValue;
	}
