#ifdef GL_ES
precision highp float;
#endif

#include "includes/common_functions.inc"
#include "includes/world_bend.inc"
#include "includes/fog.inc"
#include "includes/close_darken.inc"

attribute highp vec4 a_position;
attribute lowp vec4 a_color;
attribute lowp vec2 a_texCoord;
attribute lowp vec2 a_texCoord1;
attribute lowp vec3 a_normal;
attribute lowp vec3 a_binormal;
attribute lowp vec3 a_tangent;

// Ambient Color in RGB, intensity in A (contrast)
// Interesting fact, if u_ambientLightColor precision is set to lowp shader won't work on iphone5
uniform mediump vec4 u_ambientLightColor;
uniform highp mat4 u_modelViewProjectionMatrix;
uniform highp mat4 u_modelViewMatrix;
uniform highp mat4 u_modelMatrix;
uniform highp mat4 u_viewMatrix;

// DATA OUT TO PIXEL SHADER
varying lowp vec3 v_ambient_colour;
varying mediump vec3 v_normal;
varying mediump vec3 v_binormal;
varying mediump vec3 v_tangent;
varying lowp vec4 v_vert_color;
varying lowp vec2 v_texCoord0;
varying lowp vec2 v_texCoord1;
varying lowp vec3 v_camDir;
varying lowp vec2 v_sphericalCoords;
varying lowp float v_closeDarken;
varying lowp vec2 v_intRef;

void main() {
	v_normal = ((u_modelMatrix * vec4(a_normal, 0.0))).xyz;
	v_binormal = a_binormal;
	v_tangent = a_tangent;

	v_vert_color.rgb = vert_CalculateMRW(a_color.rgb);
	
	v_texCoord0 = a_texCoord;
	v_texCoord1 = a_texCoord1;
	
	v_camDir = vec3(u_viewMatrix[0][2], u_viewMatrix[1][2], u_viewMatrix[2][2]);
	
	lowp vec3 R = reflect(-normalize(v_camDir), normalize(v_normal));
	v_sphericalCoords = vert_CalculateSphericalCoords(v_vert_color.g, R);
	
	v_ambient_colour = vert_AmbientColor(u_ambientLightColor, v_normal.y);
	
	vec4 glpos = u_modelViewProjectionMatrix * a_position;
	
	// Set Vert colour.a to be the fog
	v_vert_color.a = clamp(depthFog(glpos.w) + heightFog(-a_position.y), 0.0, 1.0);
	
	// older values stored here for backup purposes in case we can't afford
	// or don't want to use the uniforms any more
	//v_vert_color.a = fog(glpos.w, -5.0, 0.0008, 0.2);
	//v_vert_color.a += fog(-a_position.y, 0.0, 0.0015, 1.0);
	
	// Bending
	v_closeDarken = closeDarken(glpos);
	
	// Internal Refraction UV's - Create a matrix for the surface and transpose the Eye vector into the surface matrix	
	mediump mat3 surfaceSpace = mat3(v_tangent, v_binormal*length(v_tangent),v_normal);
	vec2 ds = vec2 (surfaceSpace * normalize(v_camDir.xyz)).xy;
	v_intRef=v_texCoord1-ds;
	
	gl_Position = world_bend(u_modelViewMatrix * a_position, glpos);
}